/* global Bdf,$$ */
/**
 * Objet global définissant l'espace de nom Ficheform
 * 
 * @namespace Ficheframe,Bdf
 */
var Ficheframe = {};

Ficheframe.testToolbar = function (iframe, toolbarId, fieldeditId, editTarget, tocTarget) {
    var contentWindow = iframe.contentWindow;
    _showToolbar(false);
    var buttonId = "ficheframe_button_ficheedit";
    var pathName;
    try {
        pathName = contentWindow.location.pathname;
    } catch(error) {
        return;
    }
    if (!pathName.startsWith(_getFichothequeRoot())) {
        return;
    }
    var ficheTest = pathName.match(/\/fiches\/([a-z][a-z0-9]*)-([0-9]+)(-[_a-z0-9]+)?\.html$/);
    if (!ficheTest) {
        return;
    }
    var corpusName = ficheTest[1];
    var ficheId = ficheTest[2];
    var positions = null;
    Bdf.api.loadFiche({
        corpus: corpusName,
        id: ficheId,
        callback: function (data) {
            if (data.fiche) {
                let editable = data.fiche.editable;
                if (editable) {
                    _checkFields();
                } else {
                    $$(fieldeditId).html("");
                }
                let toc = _checkToc(contentWindow.document);
                let html = Bdf.render("ficheframe:toolbar", {
                    path: pathName,
                    corpusName: corpusName,
                    ficheId: ficheId,
                    editable: editable,
                    toc: toc,
                    editTarget: editTarget,
                    tocTarget: tocTarget
                });
                $$(toolbarId).html(html);
                if (html.length > 0) {
                    _showToolbar(true);
                    $(contentWindow).on('pagehide', function(e) {
                        _showToolbar(false);
                        _showFieldEdit(false);
                    });
                    let editLink = $$.one(buttonId);
                    let originalHref = editLink.href;
                    if (toc.length > 0) {
                        $(contentWindow).on("scroll", function(event) {
                            positions = null;
                            let threshold = contentWindow.innerHeight * 2 / 3;
                            let scrollTop = $(contentWindow.document).scrollTop();
                            let currentTocItem = false;
                            for (let i = toc.length-1; i >= 0; i--) {
                                let tocItem = toc[i];
                                if (scrollTop > $(tocItem.element).offset().top - threshold) {
                                    currentTocItem = tocItem;
                                    break;
                                }
                            }
                            $(".ficheframe-toc-Active").removeClass("ficheframe-toc-Active");
                            if (currentTocItem) {
                                $$("linkto:" + currentTocItem.id).addClass("ficheframe-toc-Active");
                                editLink.href = originalHref + "&goto=" + currentTocItem.goto;
                            } else {
                                editLink.href = originalHref;
                            }
                        });
                    }
                }
            }
        }
    });
    
    function _checkFields() {
        $(contentWindow).on("mousemove", function(event) {
            let pageY = event.pageY;
            if (!positions) {
                positions = Ficheframe.Positions.build(contentWindow.document);
            }
            if (!positions.isChanged(pageY)) {
                return;
            }
            let newPosition = positions.updatePosition(pageY);
            if ((newPosition) && (newPosition.goto)) {
                    _showFieldEdit(true);
                    newPosition.setBackground("#e5e5e5");
                    $$(fieldeditId).html(Bdf.render("ficheframe:fieldedit", {
                        corpusName: corpusName,
                        ficheId: ficheId,
                        editTarget: editTarget,
                        goto: newPosition.goto
                    })).css("top", newPosition.startY - $(contentWindow.document).scrollTop() + $(iframe).position().top) ;
            } else {
                _showFieldEdit(false);
            }
        });
        $(contentWindow).on("scroll", function() {
            if (positions) {
                positions.clear();
                _showFieldEdit(false);
            }
        });
    }
    
    function _showToolbar(display) {
        Ficheframe.show(toolbarId, display);
    }
    
    function _showFieldEdit(display) {
        Ficheframe.show(fieldeditId, display);
    }
    
    function _checkToc(document) {
        let array = new Array();
        $$(document, {bdfFormMilestone: true}).each(function (index, element) {
            let milestoneType = element.dataset.bdfFormMilestone;
            let goto;
            let $headings;
            if (milestoneType === "satellite") {
                goto = "satellite_" + element.dataset.bdfSatellite;
                $headings = $(element).children("h1");
            } else if (milestoneType === "comment") {
                goto = element.dataset.bdfComponentName;
                let $satellite = $(element).parents($$.toCssSelector({bdfFormMilestone: "satellite"}));
                if ($satellite.length > 0) {
                    goto = "satellite_" + $satellite[0].dataset.bdfSatellite + "/" + goto;
                }
                $headings = $(element).find("h2");
            }
            if ($headings) {
                $headings.each(function (index2, element2) {
                    let id = element2.id;
                    if (!id) {
                        id = Bdf.generateId();
                        element2.id = id;
                    }
                    let $element2 = $(element2);
                    let text = $element2.text();
                    if (!$element2.is(":hidden")) {
                        array.push({
                            id: id,
                            text: text,
                            element: element2,
                            goto: goto
                        });
                    }
                });
            }
        });
        return array;
    }
    
    function _getFichothequeRoot() {
        let currentPathName = window.location.pathname;
        let lastIndex = currentPathName.lastIndexOf("/");
        return currentPathName.substr(0, lastIndex + 1);
    }
};

Ficheframe.show = function (elementId, display) {
    if (display) {
        $$(elementId).removeClass("hidden");
    } else {
        $$(elementId).addClass("hidden");
    }
};

Ficheframe.Position = function (startY, endY, element, goto) {
    this.startY = startY;
    this.endY = endY;
    this.element = element;
    this.goto = goto;
    this.startMargin = startY;
    this.endMargin = endY;
};

Ficheframe.Position.prototype.contains = function (y) {
    if ((y >= this.startY) && (y <= this.endY)) {
        return true;
    } else {
        return false;
    }
};

Ficheframe.Position.prototype.containsWithMargin = function (y) {
    if ((y >= this.startMargin) && (y <= this.endMargin)) {
        return true;
    } else {
        return false;
    }
};

Ficheframe.Position.prototype.compare = function (y) {
    if (y < this.startY) {
        return -1;
    }
    if (y > this.endY) {
        return 1;
    }
    return 0;
};

Ficheframe.Position.prototype.setBackground = function (color) {
    this.element.style.backgroundColor = color;
};

Ficheframe.Positions = function (positionArray) {
  this.positionArray = positionArray; 
  this.currentIndex = -1;
  this.currentPosition = null;
};

Ficheframe.Positions.prototype.isChanged = function (pageY) {
    var currentPosition = this.currentPosition;
    if ((currentPosition) && (currentPosition.containsWithMargin(pageY))) {
        return false;
    }
    return true;
};

Ficheframe.Positions.prototype.clear = function () {
    if (this.currentPosition) {
        this.currentPosition.setBackground(null);
        this.currentPosition = null;
        this.currentIndex = -1;
    }
};

Ficheframe.Positions.prototype.updatePosition = function (pageY) {
    this.clear();
    let done = false;
    let length = this.positionArray.length;
    for(let i = 0; i < length; i++) {
        let position = this.positionArray[i];
        let comp = position.compare(pageY);
        if (comp === 0) {
            this.currentPosition = position;
            this.currentIndex = i;
            done = true;
            break;
        } else if (comp < 0) {
            break;
        }
    }
    if (done) {
        this.currentPosition.setBackground("#e5e5e5");
    }
    return this.currentPosition;
};

Ficheframe.Positions.build = function (document) {
    var positionArray = new Array();
    $(document).find($$.toCssSelector({bdfField: true})).each(function (index, element) {
        let fieldType = element.dataset.bdfField;
        let $element = $(element);
        let goto = _getGoto(element);
        let dim = _getDim($element);
        if (fieldType === "section") {
            
        } else {
            positionArray.push(new Ficheframe.Position(dim[0], dim[1], element, goto));
        }
    });
    positionArray.sort(function (p1, p2) {
        if (p1.startY < p2.startY) {
            return -1;
        }
        if (p1.startY > p2.startY) {
            return 1;
        }
        if (p1.endY < p2.endY) {
            return -1;
        }
        if (p1.endY > p2.endY) {
            return 1;
        }
        return 0;
    });
    let length = positionArray.length;
    for(let i = 0; i < length; i++) {
        let currentPosition = positionArray[i];
        if (i === 0) {
            currentPosition.startMargin = 0;
        } else {
            currentPosition.startMargin = positionArray[i -1].endY + 1;
        }
        if (i === (length - 1)) {
            currentPosition.endMargin = $(document).height();
        } else {
            currentPosition.endMargin = positionArray[i + 1].startY - 1;
        }
    }
    return new Ficheframe.Positions(positionArray);
    
    function _getDim($element) {
        let startY = $element.offset().top;
        let array = new Array();
        array.push(startY);
        array.push(startY + $element.height());
        return array;
    }
    
    function _getGoto(element) {
        let goto = element.dataset.bdfComponentName;
        if (!goto) {
            return null;
        }
        if (goto === "liage") {
            if (element.dataset.bdfField !== "corpus") {
                return null;
            }
        } else if (goto.startsWith("_master_")) {
            if (!goto.startsWith("_master_" + element.dataset.bdfField)) {
                return null;
            }
        } else if (!goto.startsWith(element.dataset.bdfField)) {
            return null;
        }
        let $satellite = $(element).parents($$.toCssSelector({bdfFormMilestone: "satellite"}));
        if ($satellite.length > 0) {
            goto = "satellite_" + $satellite[0].dataset.bdfSatellite + "/" + goto;
        }
        return goto;
    }
};
